'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var faust = require('mosfez-faust/faust');

class VoiceAllocator {
  _totalVoices = 0;
  _freeVoices = [];
  _activeVoices = [];
  _releasedVoices = [];
  constructor(totalVoices) {
    this.totalVoices = totalVoices;
  }
  get totalVoices() {
    return this._totalVoices;
  }
  set totalVoices(total) {
    this._totalVoices = total;
    this._freeVoices = [];
    this._activeVoices = [];
    this._releasedVoices = [];
    for (let i = 0; i < total; i++) {
      this._freeVoices.push(i);
    }
  }
  get freeVoices() {
    return this._freeVoices;
  }
  get activeVoices() {
    return this._activeVoices;
  }
  get releasedVoices() {
    return this._releasedVoices;
  }
  activate(id) {
    const ownVoice = this._activeVoices.find((v) => v[1] === id);
    if (ownVoice) {
      const [voice] = ownVoice;
      return voice;
    }
    const freeVoice = this._freeVoices.shift();
    if (freeVoice !== void 0) {
      this._activeVoices.push([freeVoice, id]);
      return freeVoice;
    }
    const oldestReleased = this._releasedVoices.shift();
    if (oldestReleased !== void 0) {
      const [voice, , timeoutId] = oldestReleased;
      clearTimeout(timeoutId);
      this._activeVoices.push([voice, id]);
      return voice;
    }
    const oldestActive = this._activeVoices.shift();
    if (oldestActive !== void 0) {
      const [voice] = oldestActive;
      this._activeVoices.push([voice, id]);
      return voice;
    }
    throw new Error(`Could not find active voice to reuse`);
  }
  release(id, ms) {
    const voiceIndex = this._activeVoices.findIndex((v) => v[1] === id);
    if (voiceIndex === -1)
      return -1;
    const voice = this._activeVoices[voiceIndex];
    this._activeVoices.splice(voiceIndex, 1);
    const timeoutId = setTimeout(() => this.free(id), ms);
    this._releasedVoices.push([...voice, timeoutId]);
    return voice[0];
  }
  free(id) {
    console.log("free voice", id);
    const voiceIndex = this._releasedVoices.findIndex((v) => v[1] === id);
    if (voiceIndex === -1)
      return;
    const [voice] = this._releasedVoices[voiceIndex];
    this._releasedVoices.splice(voiceIndex, 1);
    this._freeVoices.push(voice);
  }
}

console.log("compile", faust.compile);
function noteToFreq(note, rootHz) {
  return rootHz / 32 * 2 ** ((note - 9) / 12);
}
function range(total) {
  return [...Array(total)].map((_a, i) => i);
}
class MosfezXenSynth {
  _totalVoices = 8;
  _voiceAllocator = new VoiceAllocator(this._totalVoices);
  _voicesState = [];
  _rootHz = 440;
  _envRelease = 4;
  console;
  constructor() {
    this.resetVoiceState();
  }
  resetVoiceState() {
    this._voicesState = range(this._totalVoices).map(() => ({
      freq: 440,
      gate: 0
    }));
  }
  get rootHz() {
    return this._rootHz;
  }
  set rootHz(hz) {
    this._rootHz = hz;
  }
  setNote(midi, options = {}) {
    const id = options.id !== void 0 ? options.id : `${midi}`;
    const voice = this._voiceAllocator.activate(id);
    const freq = noteToFreq(midi, this._rootHz);
    const gate = 1;
    this.console?.log(`activate voice ${voice} to ${freq}`);
    const existingVoice = this._voicesState[voice];
    if (existingVoice.freq !== freq || existingVoice.gate !== gate) {
      this._voicesState[voice] = {
        freq,
        gate: 1
      };
    }
  }
  stopNote(midiOrId) {
    const id = `${midiOrId}`;
    const voice = this._voiceAllocator.release(id, this._envRelease * 1e3);
    if (voice === -1)
      return;
    this.console?.log(`release voice ${voice}`);
    this._voicesState[voice].gate = 0;
  }
}

exports.MosfezXenSynth = MosfezXenSynth;
//# sourceMappingURL=v0.js.map
